import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;

public class PolyhedronFace {
    Vector[] V=new Vector[4];
    Color FILL,OUTLINE;
    int length;

    public PolyhedronFace() {}

    public PolyhedronFace(PolyhedronFace X) {
	length=X.length;
	for(int i=0;i<length;++i) {
              V[i]=new Vector(X.V[i]);
	}
	FILL=X.FILL;
	OUTLINE=X.OUTLINE;
    }

    public void print() {
	System.out.println("-------");
	System.out.println("length "+length);
	for(int i=0;i<length;++i) V[i].print();
	System.out.println("----------------");
    }


    public static PolyhedronFace toPolyhedronFace(Polyhedron P,int choice) {
	PolyhedronFace F=new PolyhedronFace();
	F.length=P.facet[choice][0];
	for(int i=0;i<F.length;++i) {
	    F.V[i]=new Vector(P.V[P.facet[choice][i+1]]);
	}
	F.FILL=P.FILL;
	F.OUTLINE=P.OUTLINE;
	return(F);
    }



    public static PolyhedronFace[] toFaceList(Polyhedron P) {
        int len=P.facet[0][0];
	PolyhedronFace[] F=new PolyhedronFace[len];
	for(int choice=0;choice<len;++choice) {
           F[choice]=toPolyhedronFace(P,choice+1);
	}
	return(F);
    }


    public static PolyhedronFace[] toFaceListSolid(Polyhedron P,Vector[] PROJ) {
	int count=0;
        int len=P.facet[0][0];
	PolyhedronFace[] F=new PolyhedronFace[len];
	for(int choice=0;choice<len;++choice) {
           PolyhedronFace G=toPolyhedronFace(P,choice+1);
	   int test=testOrientation(G,PROJ);
	   if(test==1) {
	       F[count]=new PolyhedronFace(G);
	       ++count;
	   }
	}
	PolyhedronFace[] FF=new PolyhedronFace[count];
	for(int i=0;i<count;++i) FF[i]=new PolyhedronFace(F[i]);
	return(FF);
    }



    public static PolyhedronFace[] toFaceListTotal(Polyhedron P,Vector[] PROJ) {
	int count=0;
        int len=P.facet[0][0];
	PolyhedronFace[] F=new PolyhedronFace[len];

	for(int choice=0;choice<len;++choice) {
           PolyhedronFace G=toPolyhedronFace(P,choice+1);
	   int test=testOrientation(G,PROJ);
	   if(test!=1) {
	       F[count]=new PolyhedronFace(G);
	       ++count;
	   }
	}
	for(int choice=0;choice<len;++choice) {
           PolyhedronFace G=toPolyhedronFace(P,choice+1);
	   int test=testOrientation(G,PROJ);
	   if(test==1) {
	       F[count]=new PolyhedronFace(G);
	       ++count;
	   }
	}

	PolyhedronFace[] FF=new PolyhedronFace[count];
	for(int i=0;i<count;++i) FF[i]=new PolyhedronFace(F[i]);
	return(FF);
    }





    public GeneralPathColor toPath(Vector[] PROJ) {
	GeneralPath gp=new GeneralPath();
	for(int i=0;i<length;++i) {
	    Complex z=V[i].project(PROJ);
	       if(i==0) gp.moveTo((float)(z.x),(float)(z.y));
	       if(i> 0) gp.lineTo((float)(z.x),(float)(z.y));
	}
	gp.closePath();
	GeneralPathColor gpc=new GeneralPathColor(gp,FILL,OUTLINE);
	return(gpc);
    }


    public static GeneralPathColor[] toPathListSolid(Polyhedron P,Vector[] PROJ) {
	PolyhedronFace[] F=toFaceListSolid(P,PROJ);
	GeneralPathColor[] gp=new GeneralPathColor[F.length];
	for(int i=0;i<F.length;++i) {

	    gp[i]=F[i].toPath(PROJ);
	}
	return(gp);
    }


    public static GeneralPathColor[] toPathListTotal(Polyhedron P,Vector[] PROJ) {
	PolyhedronFace[] F=toFaceListTotal(P,PROJ);
	GeneralPathColor[] gp=new GeneralPathColor[F.length];
	for(int i=0;i<F.length;++i) {

	    gp[i]=F[i].toPath(PROJ);
	}
	return(gp);
    }








    /**this works for triangular faces*/


    public static Vector fromBarycenter(double[] t,PolyhedronFace F) {
	Vector V=new Vector();
	for(int i=0;i<3;++i) {
	    for(int j=0;j<3;++j) {
		V.x[j]=V.x[j]+t[i]*F.V[i].x[j];
	    }
	}
	return(V);
    }

    public Facet toFacet(Vector[] PROJ) {
	Facet F=new Facet();
	for(int i=0;i<3;++i) {
	    Complex z=V[i].project(PROJ);
	    F.z[i]=z;
	}
	return(F);
    }

    public static int top0(PolyhedronFace V1,PolyhedronFace V2,Vector[] PROJ) {
	Facet F1=V1.toFacet(PROJ);
	Facet F2=V2.toFacet(PROJ);
	if(Facet.testEquals(F1,F2)==1) return(0);
	int test=Facet.testDisjoint(F1,F2);
	if(test==1) return(-1);
	double[] t=Facet.goodBarycentricCoords(F1,F2);
	double[] t1={t[0],t[1],t[2]};
	double[] t2={t[3],t[4],t[5]};

	Vector X1=fromBarycenter(t1,V1);
	Vector X2=fromBarycenter(t2,V2);
	double d1=Vector.dot(X1,PROJ[0]);
	double d2=Vector.dot(X2,PROJ[0]);
	if(d1<d2) return(0);
	return(1);
    }

    /* subdivide into triangles*/

   public static PolyhedronFace[] subdivide(PolyhedronFace V) {
       if(V.length==3) {
	   PolyhedronFace[] X={V};
	   return(X);
       }
	PolyhedronFace V1=new PolyhedronFace();
	PolyhedronFace V2=new PolyhedronFace();
	V1.length=3;
	V2.length=3;

	V1.V[0]=V.V[0];
	V1.V[1]=V.V[1];
	V1.V[2]=V.V[2];

	V2.V[0]=V.V[0];
	V2.V[1]=V.V[2];
	V2.V[2]=V.V[3];
	PolyhedronFace[] X={V1,V2};
	return(X);
    }

    public static int top(PolyhedronFace V1,PolyhedronFace V2,Vector[] PROJ) {
	PolyhedronFace[] X1=subdivide(V1);
	PolyhedronFace[] X2=subdivide(V2);
	for(int i=0;i<X1.length;++i) {
	    for(int j=0;j<X2.length;++j) {
		int test=top0(X1[i],X2[j],PROJ);
		if(test==1) return(1);
	    }
	}
	return(0);
    }



    public static int top(Polyhedron P1,Polyhedron P2,Vector[] PROJ) {
	PolyhedronFace[] F1=toFaceListSolid(P1,PROJ);
	PolyhedronFace[] F2=toFaceListSolid(P2,PROJ);
	for(int i=0;i<F1.length;++i) {
	    for(int j=0;j<F2.length;++j) {
		int test=top(F1[i],F2[j],PROJ);
		if(test==1) return(1);
	    }
	}
	return(0);
    }


    public static int testOrientation(PolyhedronFace FF,Vector[] PROJ) {
	PolyhedronFace[] X=subdivide(FF);
	Facet f=X[0].toFacet(PROJ);
	double a=Complex.signedArea(f.z[0],f.z[1],f.z[2]);
	if(a<0) return(1);
	return(0);
    }


}

