import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


/*This class does the basic arithmetic
  of vectors */


public class Vector {
    double[] x=new double[3];
    public Vector(){}

    public Vector(Complex z) {
	x[0]=z.x;
	x[1]=z.y;
	x[2]=1.0;
    }
    public Vector(Vector V) {
	for(int i=0;i<3;++i) x[i]=V.x[i];
    }

    public Vector(double xx,double yy) {
	x[0]=xx;
	x[1]=yy;
	x[2]=1.0;
    }

    public Vector(double xx,double yy,double zz) {
	x[0]=xx;
	x[1]=yy;
	x[2]=zz;
    }

    public static double dist(Vector W1,Vector W2) {
	double xx=W1.x[0]-W2.x[0];
	double yy=W1.x[1]-W2.x[1];
	double zz=W1.x[2]-W2.x[2];
	double d=xx*xx+yy*yy+zz*zz;
	d=Math.sqrt(d);
	return(d);
    }

    public static Vector average(Vector V,Vector W) {
	Vector X=new Vector();
	for(int i=0;i<3;++i) X.x[i]=.5*(V.x[i]+W.x[i]);
	return(X);
    }


    public static double dist2(Vector W1,Vector W2) {
	double xx=W1.x[0]-W2.x[0];
	double yy=W1.x[1]-W2.x[1];
	double zz=W1.x[2]-W2.x[2];
	double d=xx*xx+yy*yy+zz*zz;
	return(d);
    }

    public double norm() {
	return(dist(this,new Vector()));
    }


    public static double  dot(Vector v,Vector w) {
	return(v.x[0]*w.x[0]+v.x[1]*w.x[1]+v.x[2]*w.x[2]);
    }

    public static Vector cross(Vector v,Vector w) {
	Vector X=new Vector();
	X.x[0]=v.x[1]*w.x[2]-w.x[1]*v.x[2];
	X.x[1]=v.x[2]*w.x[0]-w.x[2]*v.x[0];
	X.x[2]=v.x[0]*w.x[1]-w.x[0]*v.x[1];
	return(X);
    }

    public static Vector plus(Vector v1,Vector v2) {
	Vector w=new Vector();
	for(int i=0;i<3;++i) {
	    w.x[i]=v1.x[i]+v2.x[i];
	}
	return(w);
    }

    public static Vector minus(Vector v1,Vector v2) {
	Vector w=new Vector();
	for(int i=0;i<3;++i) {
	    w.x[i]=v1.x[i]-v2.x[i];
	}
	return(w);
    }


    public static Vector interpolate(Vector V,Vector W,double t) {
	Vector V2=V.scale(1-t);
	Vector W2=W.scale(t);
	return plus(V2,W2);
    }

    public void print() {
	System.out.println(x[0]+" "+x[1]+" "+x[2]);
    }

    public Vector scale(double d) {
	Vector V=new Vector(d*x[0],d*x[1],d*x[2]);
	return(V);
    }

    public Vector unit() {
	double d=dot(this,this);
	d=Math.sqrt(d);
	Vector W=new Vector(x[0]/d,x[1]/d,x[2]/d);
	return(W);
    }


    public Complex  toComplex() {
	return(new Complex(x[0]/x[2],x[1]/x[2]));
    }

    public static Vector midVector(Vector V1,Vector V2) {
	Vector W1=V1.unit();
	Vector W2=V2.unit();
	Vector W3=minus(W1,W2);
	Vector W4=W3.unit();
	return(W4);
    }

    public static double tripleProduct(Vector v1,Vector v2,Vector v3) {
	return(dot(cross(v1,v2),v3));
    }

    public static double angle(Vector V1,Vector V2) {
	double d1=dot(V1,V2);
	double d2=dot(V1,V1);
	double d3=dot(V2,V2);
	double d=d1*d1/(d2*d3);
	d=Math.sqrt(d);
	d=Math.acos(d);
	return(d);
    }

    public static Vector findCross(Vector v1,Vector v2,Vector v3,Vector v4) {
	Vector v5=cross(v1,v2);
	Vector v6=cross(v3,v4);
	Vector v7=cross(v5,v6);
	return(v7);
    }


    //Gram-Schmidt

    public Vector proj(Vector W) {
	Vector V1=this.unit();
	double d=dot(V1,W);
	Vector V2=V1.scale(d);
	Vector X=minus(W,V2);
	return(X);
    }

    //rotates about (this) vector, fixing the origin

    public Vector rotate(Vector W) {
	Vector V1=this.unit();
	double d=dot(V1,W);
	Vector V2=V1.scale(2*d);
	Vector X=minus(V2,W);
	return(X);
    }

    //rotates in space about V2, fixing V1

    public static Vector rotate(Vector V1,Vector V2,Vector W) {
	Vector W2=Vector.minus(W,V1);
	Vector W3=V2.rotate(W2);
	Vector W4=Vector.plus(W3,V1);
	return(W4);
    }




    /**Given a segment V1V2, and a vector W not on that segment,
       this routine returns the nearest point on V1V2 to W.  The
       routine is only invoked when this nearest point is known
       in advance to be interior point*/

    public static Vector nearestPointProjection(Vector V1,Vector V2,Vector W) {
	Vector X1=minus(W,V1);
	Vector X2=minus(V2,V1);
	double t=dot(X1,X2)/dot(X2,X2);
	Vector Y=Vector.plus(V1,X2.scale(t));
	return(Y);
    }



    public static Vector specialCross(Vector V1,Vector V2,Vector V3,Vector V4,Vector V5) {

	double d1,d2,d3,m1,m2,m3,e1,e2,j1,j2,k1,k2,k3;
	d1=V1.x[0];
	d2=V1.x[2];
	d3=V1.x[1];	
        m1=V2.x[0];
	m2=V2.x[2];
	m3=V2.x[1];
	e1=V3.x[0];
	e2=V3.x[2];
	j1=V4.x[0];
	j2=V4.x[2];
        k1=V5.x[0];
	k2=V5.x[2];
	k3=V5.x[1];

	double s1=-e2*j1+e1*j2+e2*m1 - j2*m1 - e1*m2 + j1*m2;
	double s2=d2*e1-d1*e2-d2*j1+d1*j2+e2*m1-j2*m1-e1*m2+j1*m2;
	double s=s1/s2;


	double p1=s*d1+ (1-s)*k1;
	double p2=s*d2+ (1-s)*k2;
	double p3=s*d3+ (1-s)*k3;

	Vector X=new Vector(p1,p3,p2);
	return(X);
    }


    public GeneralPath toPlan(double d) {
	GeneralPath gp=new GeneralPath();
	gp.moveTo((float)(x[0]-d),(float)(x[1]-d));
	gp.lineTo((float)(x[0]-d),(float)(x[1]+d));
	gp.lineTo((float)(x[0]+d),(float)(x[1]+d));
	gp.lineTo((float)(x[0]+d),(float)(x[1]-d));
	gp.closePath();
	return(gp);
    }

  


    public Complex project(Vector[] V) {
	Matrix m=Matrix.fromFrameInverse(V[0],V[1]);
	Vector W=Matrix.act(m,this);
	Complex z=new Complex(W.x[0],W.x[1]);
	return(z);
    }


    public static Complex f1(Complex z) {
	double a= 2.1973682269356199;
	double b= -1.0986841134678099660;
	double c= 1.2871885058111652495;
        double d= -0.18850439234335528343;
	double t1=z.norm();
	double t2=(a*t1+b)/(c*t1+d);
	Complex w=new Complex(t2*z.x/t1,t2*z.y/t1);
	return(w);
    }



    public static Vector[] hemisphere(Complex Z) {
	Complex z1 = new Complex(Z);
	Complex z2=f1(Z);

	double t1=.5-z2.norm()*z2.norm();
	double t=0;
	Vector V=new Vector();

	if(t1>0) {  
	    t1=2*t1;
            double t2=Math.sqrt(1-t1*t1);
            V=new Vector(t2*z2.x,-t2*z2.y,t1);
	}

        if(t1<=0) {
	    t1=2*t1;
	    if(t1<-.999) t1=-1;
	    double t2=Math.sqrt(1-t1*t1);  


	    V=new Vector(t2*z2.x,-t2*z2.y,t1);
	}

	z1=new Complex(z1.y,-z1.x);
	z1=z1.unit();
	Vector W=new Vector(z1.x,-z1.y,0);
	W=cross(V,W);
	W=cross(W,V);
	W=W.unit();
	V=V.unit();
	Vector[] X={V,W};
	return(X);
    }






}




